@@ -4,19 +4,24 @@
  *	file accessing module
  */
 
-#include <fcntl.h>
 #include "fd.h"
+#include "time.h"
 #include "termio.h"
+#include "realpath.h"
+#include "kconv.h"
 #include "func.h"
 #include "kanji.h"
 
+#ifdef	DEP_URLPATH
+#include "urldisk.h"
+#endif
 #if	MSDOS
 #include <process.h>
 #endif
 
 #define	MAXTMPNAMLEN		8
 #define	LOCKEXT			"LCK"
-#ifdef	_NODOSDRIVE
+#ifndef	DEP_DOSDRIVE
 #define	DOSDIRENT		32
 #define	LFNENTSIZ		13
 #endif
@@ -28,33 +33,10 @@
 #define	FAT_LFN			2
 #define	FAT_DOSDRIVE		3
 
-#ifndef	O_BINARY
-#define	O_BINARY		0
-#endif
-#ifndef	O_ACCMODE
-#define	O_ACCMODE		(O_RDONLY | O_WRONLY | O_RDWR)
-#endif
-#ifndef	ENOSPC
-#define	ENOSPC			EACCES
-#endif
-#ifndef	EIO
-#define	EIO			ENODEV
-#endif
-#ifndef	ETIMEDOUT
-#define	ETIMEDOUT		EIO
-#endif
-#ifndef	L_SET
-# ifdef	SEEK_SET
-# define	L_SET		SEEK_SET
-# else
-# define	L_SET		0
-# endif
-#endif	/* !L_SET */
-
 #if	MSDOS
 extern int getcurdrv __P_((VOID_A));
 #endif
-#if	MSDOS && !defined (_NOUSELFN) && !defined (_NODOSDRIVE)
+#if	defined (DEP_DOSDRIVE) && defined (DEP_DOSLFN)
 extern int checkdrive __P_((int));
 #endif
 
@@ -62,25 +44,26 @@ extern int mark;
 extern char fullpath[];
 extern char *tmpfilename;
 extern int physical_path;
-extern int noconv;
-#ifndef	_NODOSDRIVE
-extern int dosdrive;
+#ifdef	DEP_PSEUDOPATH
+extern char *unixpath;
 #endif
 
 #ifndef	NOFLOCK
 static int NEAR fcntllock __P_((int, int));
 #endif
 static char *NEAR excllock __P_((CONST char *, int));
-#ifdef	_NODOSDRIVE
-#define	nodoschdir		Xchdir
-#define	nodosgetwd		Xgetwd
-#define	nodosmkdir		Xmkdir
-#define	nodosrmdir		Xrmdir
+#ifdef	DEP_PSEUDOPATH
+static int NEAR pushswap __P_((VOID_A));
+static VOID NEAR popswap __P_((int));
+static int NEAR realchdir __P_((CONST char *));
+static char *NEAR realgetwd __P_((char *));
+static int NEAR realmkdir __P_((CONST char *, int));
+static int NEAR realrmdir __P_((CONST char *));
 #else
-static int NEAR nodoschdir __P_((CONST char *));
-static char *NEAR nodosgetwd __P_((char *));
-static int NEAR nodosmkdir __P_((CONST char *, int));
-static int NEAR nodosrmdir __P_((CONST char *));
+#define	realchdir		Xchdir
+#define	realgetwd		Xgetwd
+#define	realmkdir		Xmkdir
+#define	realrmdir		Xrmdir
 #endif
 static int NEAR cpfile __P_((CONST char *, CONST char *,
 		struct stat *, struct stat *));
@@ -107,12 +90,21 @@ int inheritcopy = 0;
 #endif
 int tmpumask = TMPUMASK;
 
-#ifndef	_NODOSDRIVE
-static int dosdrv = -1;
-#endif
+#ifdef	DEP_PSEUDOPATH
+static int pseudodrv = -1;
+static int *swaplist[] = {
+# ifdef	DEP_DOSDRIVE
+	&dosdrive,
+# endif
+# ifdef	DEP_URLPATH
+	&urldrive,
+# endif
+};
+#define	SWAPLISTSIZ		arraysize(swaplist)
+#endif	/* DEP_PSEUDOPATH */
 
 
-#ifdef	_USEDOSEMU
+#ifdef	DEP_DOSEMU
 CONST char *nodospath(path, file)
 char *path;
 CONST char *file;
@@ -120,41 +112,16 @@ CONST char *file;
 	if (!_dospath(file)) return(file);
 	path[0] = '.';
 	path[1] = _SC_;
-	strcpy(&(path[2]), file);
+	Xstrcpy(&(path[2]), file);
 
 	return(path);
 }
-#endif	/* _USEDOSEMU */
-
-#ifdef	NOUID
-int logical_access(mode)
-u_int mode;
-#else
-int logical_access(mode, uid, gid)
-u_int mode;
-uid_t uid;
-gid_t gid;
-#endif
-{
-	int dir;
-
-	dir = ((mode & S_IFMT) == S_IFDIR);
-#ifdef	NOUID
-	mode >>= 6;
-#else	/* !NOUID */
-	if (uid == geteuid()) mode >>= 6;
-	else if (gid == getegid()) mode >>= 3;
-	else if (isgroupmember(gid)) mode >>= 3;
-#endif	/* !NOUID */
-	if (dir && !(mode & F_ISEXE)) mode &= ~(F_ISRED | F_ISWRI);
-
-	return(mode & 007);
-}
+#endif	/* DEP_DOSEMU */
 
 int getstatus(namep)
 namelist *namep;
 {
-#ifdef	_USEDOSEMU
+#ifdef	DEP_DOSEMU
 	char path[MAXPATHLEN];
 #endif
 	struct stat st, lst;
@@ -169,9 +136,7 @@ namelist *namep;
 	if (isdisplnk(dispmode))
 		memcpy((char *)&lst, (char *)&st, sizeof(struct stat));
 #if	!MSDOS
-	if ((lst.st_mode & S_IFMT) == S_IFCHR
-	|| (lst.st_mode & S_IFMT) == S_IFBLK)
-		namep -> flags |= F_ISDEV;
+	if (s_ischr(&lst) || s_isblk(&lst)) namep -> flags |= F_ISDEV;
 #endif
 
 	namep -> st_mode = lst.st_mode;
@@ -295,9 +260,9 @@ CONST VOID_P vp2;
 			tmp = strpathcmp2(tp1 -> name, tp2 -> name);
 			break;
 		case 2:
-			if ((cp1 = strrchr(tp1 -> name, '.'))) cp1++;
+			if ((cp1 = Xstrrchr(tp1 -> name, '.'))) cp1++;
 			else cp1 = nullstr;
-			if ((cp2 = strrchr(tp2 -> name, '.'))) cp2++;
+			if ((cp2 = Xstrrchr(tp2 -> name, '.'))) cp2++;
 			else cp2 = nullstr;
 			tmp = strpathcmp2(cp1, cp2);
 			break;
@@ -363,14 +328,17 @@ char *buf;
 	}
 	if (!homedir) {
 		if (!(cp = gethomedir())) return(-1);
-#ifndef	_NODOSDRIVE
+#ifdef	DEP_DOSDRIVE
 		if (dospath2(cp)) return(-1);
+#endif
+#ifdef	DEP_URLPATH
+		if (urlpath(cp, NULL, NULL, NULL)) return(-1);
 #endif
 		if (getrealpath(cp, tmp, cwd) != tmp) return(-1);
-		homedir = strdup2(tmp);
+		homedir = Xstrdup(tmp);
 	}
 
-	if (buf && !physical_path) strcpy(cwd, fullpath);
+	if (buf && !physical_path) Xstrcpy(cwd, fullpath);
 	len = strlen(homedir);
 #if	MSDOS
 	if (len <= 3) cp = NULL;
@@ -379,9 +347,9 @@ char *buf;
 #endif
 	else cp = underpath(cwd, homedir, len);
 
-	if (buf) strcpy(buf, (cp) ? cp : nullstr);
+	if (buf) Xstrcpy(buf, (cp) ? cp : nullstr);
 #ifdef	DEBUG
-	free(homedir);
+	Xfree(homedir);
 	homedir = NULL;
 #endif
 
@@ -396,7 +364,7 @@ CONST char *dir;
 	CONST char *cp;
 
 	cp = dir;
-#ifdef	_USEDOSPATH
+#ifdef	DEP_DOSPATH
 	if (_dospath(dir)) cp += 2;
 #endif
 	if (!isdotdir(cp)) {
@@ -416,55 +384,27 @@ CONST char *dir;
 			}
 			else if (stat2(dir, &st) < 0) return(-1);
 		}
-		if (!s_isdir(&st)) {
-			errno = ENOTDIR;
-			return(-1);
-		}
+		if (!s_isdir(&st)) return(seterrno(ENOTDIR));
 	}
-	entryhist(1, realpath2(dir, tmp, 0), 1);
+	entryhist(Xrealpath(dir, tmp, 0), HST_PATH | HST_UNIQ);
 
 	return(0);
 }
 
 #ifndef	NOFLOCK
-static int NEAR fcntllock(fd, mode)
-int fd, mode;
+static int NEAR fcntllock(fd, operation)
+int fd, operation;
 {
-	static int lockmode[] = {
-# ifdef	USEFCNTLOCK
-		F_RDLCK, F_WRLCK, F_UNLCK,
-# else	/* !USEFCNTLOCK */
-#  ifdef	USELOCKF
-		F_TLOCK, F_TLOCK, F_ULOCK,
-#  else
-		LOCK_SH | LOCK_NB, LOCK_EX | LOCK_NB, LOCK_UN,
-#  endif
-# endif	/* !USEFCNTLOCK */
-	};
-# ifdef	USEFCNTLOCK
-	struct flock lock;
-# endif
 	int i, n;
 
-# ifndef	_NODOSDRIVE
-	if (fd >= DOSFDOFFSET) return(0);
+# ifdef	DEP_PSEUDOPATH
+	if (chkopenfd(fd) != DEV_NORMAL) return(1);
 # endif
 
 	n = -1;
 	errno = 0;
 	for (i = 0; i < LCK_MAXRETRY; i++) {
-# ifdef	USEFCNTLOCK
-		lock.l_type = lockmode[mode];
-		lock.l_start = lock.l_len = (off_t)0;
-		lock.l_whence = SEEK_SET;
-		n = fcntl(fd, F_SETLK, &lock);
-# else	/* !USEFCNTLOCK */
-#  ifdef	USELOCKF
-		n = lockf(fd, lockmode[mode], (off_t)0);
-#  else
-		n = flock(fd, lockmode[mode]);
-#  endif
-# endif	/* !USEFCNTLOCK */
+		n = Xflock(fd, operation);
 		if (n >= 0) {
 			n = 1;
 			break;
@@ -494,9 +434,9 @@ int fd, mode;
 }
 #endif	/* !NOFLOCK */
 
-static char *NEAR excllock(file, mode)
+static char *NEAR excllock(file, operation)
 CONST char *file;
-int mode;
+int operation;
 {
 #if	MSDOS
 	char *ext;
@@ -510,14 +450,14 @@ int mode;
 		if (locklist) {
 			for (i = 0; locklist[i]; i++) {
 				VOID_C unlink(locklist[i]);
-				free(locklist[i]);
+				Xfree(locklist[i]);
 			}
-			free(locklist);
+			Xfree(locklist);
 		}
 		return(NULL);
 	}
 
-	if (mode == LCK_UNLOCK) {
+	if (operation & LOCK_UN) {
 		VOID_C Xunlink(file);
 		if (locklist) {
 			for (i = 0; locklist[i]; i++)
@@ -528,7 +468,7 @@ int mode;
 				locklist[i] = NULL;
 			}
 			if (!i) {
-				free(locklist);
+				Xfree(locklist);
 				locklist = NULL;
 			}
 		}
@@ -542,23 +482,20 @@ int mode;
 	if (i > DOSBODYLEN - strsize(LOCKEXT))
 		i = DOSBODYLEN - strsize(LOCKEXT);
 	i += cp - file;
-	snprintf2(path, sizeof(path), "%-.*s%s%s", i, file, LOCKEXT, ext);
+	VOID_C Xsnprintf(path, sizeof(path),
+		"%-.*s%s%s", i, file, LOCKEXT, ext);
 #else
-	snprintf2(path, sizeof(path), "%s.%s", file, LOCKEXT);
+	VOID_C Xsnprintf(path, sizeof(path), "%s.%s", file, LOCKEXT);
 #endif
 
 	fd = -1;
 	for (i = 0; i < LCK_MAXRETRY; i++) {
-		if (mode != LCK_READ)
+		if (!(operation & LOCK_SH))
 			fd = Xopen(path,
 				O_BINARY | O_WRONLY | O_CREAT | O_EXCL,
 				0666 & ~tmpumask);
-		else if (Xlstat(path, &st) < 0)
-			fd = (errno == ENOENT) ? 0 : -1;
-		else {
-			errno = EEXIST;
-			fd = -1;
-		}
+		else if (Xlstat(path, &st) >= 0) fd = seterrno(EEXIST);
+		else fd = (errno == ENOENT) ? 0 : -1;
 		if (fd >= 0 || errno != EEXIST) break;
 
 		if (intrkey(-1)) {
@@ -571,12 +508,12 @@ int mode;
 	}
 	if (i >= LCK_MAXRETRY) return(vnullstr);
 	if (fd < 0) return(NULL);
-	if (mode == LCK_READ) return((char *)file);
+	if (operation & LOCK_SH) return((char *)file);
 
-	Xclose(fd);
-	cp = strdup2(path);
+	VOID_C Xclose(fd);
+	cp = Xstrdup(path);
 	i = countvar(locklist);
-	locklist = (char **)realloc2(locklist, (i + 2) * sizeof(char *));
+	locklist = (char **)Xrealloc(locklist, (i + 2) * sizeof(char *));
 	locklist[i] = cp;
 	locklist[++i] = NULL;
 
@@ -592,43 +529,50 @@ int flags, mode;
 #endif
 	char *lckname;
 	lockbuf_t *lck;
-	int fd, err, lckflags, lckmode, duperrno;
+	int fd, err, lckflags, operation, duperrno;
 
 #ifdef	FAKEUNINIT
 	fd = -1;	/* fake for -Wuninitialized */
 #endif
 	lckname = NULL;
 	lckflags = 0;
-	lckmode = ((flags & O_ACCMODE) == O_RDONLY) ? LCK_READ : LCK_WRITE;
+	operation = ((flags & O_ACCMODE) == O_RDONLY) ? LOCK_SH : LOCK_EX;
+	operation |= LOCK_NB;
 	err = 0;
 
 #ifndef	NOFLOCK
 	if (isnfs(path) <= 0) {
 		lckflags |= LCK_FLOCK;
+# ifdef	NOFTRUNCATE
+		fd = newdup(Xopen(path, flags, mode));
+# else
 		fd = newdup(Xopen(path, flags & ~O_TRUNC, mode));
+# endif
 		if (fd < 0) {
 			if ((flags & O_ACCMODE) == O_WRONLY || errno != ENOENT)
 				return(NULL);
 		}
-		else if ((n = fcntllock(fd, lckmode)) <= 0) {
-			Xclose(fd);
+		else if ((n = fcntllock(fd, operation)) <= 0) {
+			VOID_C Xclose(fd);
 			if (n < 0) return(NULL);
 			err++;
 			lckflags &= ~LCK_FLOCK;
 		}
+# ifndef	NOFTRUNCATE
 		else if ((flags & O_TRUNC) && Xftruncate(fd, (off_t)0) < 0) {
 			duperrno = errno;
-			VOID_C fcntllock(fd, LCK_UNLOCK);
-			Xclose(fd);
+			VOID_C fcntllock(fd, LOCK_UN);
+			VOID_C Xclose(fd);
 			errno = duperrno;
 			return(NULL);
 		}
+# endif
 	}
 #endif	/* !NOFLOCK */
 
 	if (!(lckflags & LCK_FLOCK)) {
-		if (!(lckname = excllock(path, lckmode))) return(NULL);
-		else if (lckmode == LCK_READ) {
+		if (!(lckname = excllock(path, operation))) return(NULL);
+		else if (operation & LOCK_SH) {
 			if (lckname == vnullstr) err++;
 			lckname = NULL;
 		}
@@ -638,8 +582,8 @@ int flags, mode;
 		else if ((flags & O_ACCMODE) == O_WRONLY || errno != ENOENT) {
 			duperrno = errno;
 			if (lckname) {
-				VOID_C excllock(lckname, LCK_UNLOCK);
-				free(lckname);
+				VOID_C excllock(lckname, LOCK_UN);
+				Xfree(lckname);
 			}
 			errno = duperrno;
 			return(NULL);
@@ -648,11 +592,11 @@ int flags, mode;
 
 	if (fd < 0) lckflags |= LCK_INVALID;
 	else if (err) {
-		fprintf2(stderr, "%k: %k\r\n", path, NOLCK_K);
+		VOID_C Xfprintf(Xstderr, "%k: %k\r\n", path, NOLCK_K);
 		if (isttyiomode) warning(0, HITKY_K);
 	}
 
-	lck = (lockbuf_t *)malloc2(sizeof(lockbuf_t));
+	lck = (lockbuf_t *)Xmalloc(sizeof(lockbuf_t));
 	lck -> fd = fd;
 	lck -> fp = NULL;
 	lck -> name = lckname;
@@ -666,7 +610,7 @@ CONST char *path, *type;
 int flags;
 {
 	lockbuf_t *lck;
-	FILE *fp;
+	XFILE *fp;
 
 	if (!(lck = lockopen(path, flags, 0666))) return(NULL);
 
@@ -687,19 +631,20 @@ lockbuf_t *lck;
 {
 	if (lck) {
 		if (lck -> name) {
-			VOID_C excllock(lck -> name, LCK_UNLOCK);
-			free(lck -> name);
+			VOID_C excllock(lck -> name, LOCK_UN);
+			Xfree(lck -> name);
 		}
 		if (!(lck -> flags & LCK_INVALID)) {
 #ifndef	NOFLOCK
 			if (lck -> flags & LCK_FLOCK)
-				VOID_C fcntllock(lck -> fd, LCK_UNLOCK);
+				VOID_C fcntllock(lck -> fd, LOCK_UN);
 #endif
-			if (lck -> flags & LCK_STREAM) Xfclose(lck -> fp);
-			else Xclose(lck -> fd);
+			if (lck -> flags & LCK_STREAM)
+				VOID_C Xfclose(lck -> fp);
+			else VOID_C Xclose(lck -> fd);
 		}
 
-		free(lck);
+		Xfree(lck);
 	}
 }
 
@@ -707,14 +652,14 @@ int touchfile(path, stp)
 CONST char *path;
 struct stat *stp;
 {
-#ifdef	USEUTIME
-	struct utimbuf times;
-#else
-	struct timeval tvp[2];
-#endif
 #ifndef	_NOEXTRAATTR
 	int i;
 #endif
+#ifndef	NOUID
+	u_id_t uid;
+	g_id_t gid;
+#endif
+	struct utimes_t ut;
 	struct stat st;
 	u_int mode;
 	int ret, duperrno;
@@ -733,21 +678,11 @@ struct stat *stp;
 			stp -> st_atime = st.st_atime;
 		if (!(stp -> st_nlink & TCH_MTIME))
 			stp -> st_mtime = st.st_mtime;
-#ifdef	USEUTIME
-		times.actime = stp -> st_atime;
-		times.modtime = stp -> st_mtime;
+		ut.actime = stp -> st_atime;
+		ut.modtime = stp -> st_mtime;
 		duperrno = errno;
-		if (Xutime(path, &times) >= 0) errno = duperrno;
+		if (Xutimes(path, &ut) >= 0) errno = duperrno;
 		else ret = -1;
-#else
-		tvp[0].tv_sec = stp -> st_atime;
-		tvp[0].tv_usec = 0;
-		tvp[1].tv_sec = stp -> st_mtime;
-		tvp[1].tv_usec = 0;
-		duperrno = errno;
-		if (Xutimes(path, tvp) >= 0) errno = duperrno;
-		else ret = -1;
-#endif
 	}
 
 #ifdef	HAVEFLAGS
@@ -793,11 +728,12 @@ struct stat *stp;
 	if (stp -> st_nlink & (TCH_UID | TCH_GID)) {
 		if (!(stp -> st_nlink & TCH_UID)) stp -> st_uid = (uid_t)-1;
 		if (!(stp -> st_nlink & TCH_GID)) stp -> st_gid = (gid_t)-1;
+		uid = convuid(stp -> st_uid);
+		gid = convgid(stp -> st_gid);
 		duperrno = errno;
-		if (Xchown(path, stp -> st_uid, stp -> st_gid) >= 0)
-			errno = duperrno;
+		if (Xchown(path, uid, gid) >= 0) errno = duperrno;
 		else if (!(stp -> st_nlink & TCH_CHANGE)) {
-			Xchown(path, (uid_t)-1, stp -> st_gid);
+			Xchown(path, (u_id_t)-1, gid);
 			errno = duperrno;
 		}
 		else ret = -1;
@@ -808,75 +744,95 @@ struct stat *stp;
 	return(ret);
 }
 
-#ifndef	_NODOSDRIVE
-static int NEAR nodoschdir(path)
+#ifdef	DEP_PSEUDOPATH
+static int NEAR pushswap(VOID_A)
+{
+	int i, flags;
+
+	flags = 0;
+	for (i = 0; i < SWAPLISTSIZ; i++) {
+		flags <<= 2;
+		flags |= (*(swaplist[i]) & 3);
+		*(swaplist[i]) = 0;
+	}
+
+	return(flags);
+}
+
+static VOID NEAR popswap(flags)
+int flags;
+{
+	int i;
+
+	for (i = SWAPLISTSIZ - 1; i >= 0; i--) {
+		*(swaplist[i]) = (flags & 3);
+		flags >>= 2;
+	}
+}
+
+static int NEAR realchdir(path)
 CONST char *path;
 {
-	int n, dupdosdrive;
+	int n, flags;
 
-	dupdosdrive = dosdrive;
-	dosdrive = 0;
+	flags = pushswap();
 	n = Xchdir(path);
-	dosdrive = dupdosdrive;
+	popswap(flags);
 
 	return(n);
 }
 
-static char *NEAR nodosgetwd(path)
+static char *NEAR realgetwd(path)
 char *path;
 {
 	char *cp;
-	int dupdosdrive;
+	int flags;
 
-	dupdosdrive = dosdrive;
-	dosdrive = 0;
+	flags = pushswap();
 	cp = Xgetwd(path);
-	dosdrive = dupdosdrive;
+	popswap(flags);
 
 	return(cp);
 }
 
-int nodoslstat(path, stp)
+int reallstat(path, stp)
 CONST char *path;
 struct stat *stp;
 {
-	int n, dupdosdrive;
+	int n, flags;
 
-	dupdosdrive = dosdrive;
-	dosdrive = 0;
+	flags = pushswap();
 	n = Xlstat(path, stp);
-	dosdrive = dupdosdrive;
+	popswap(flags);
 
 	return(n);
 }
 
-static int NEAR nodosmkdir(path, mode)
+static int NEAR realmkdir(path, mode)
 CONST char *path;
 int mode;
 {
-	int n, dupdosdrive;
+	int n, flags;
 
-	dupdosdrive = dosdrive;
-	dosdrive = 0;
+	flags = pushswap();
 	n = Xmkdir(path, mode);
-	dosdrive = dupdosdrive;
+	popswap(flags);
 
 	return(n);
 }
 
-static int NEAR nodosrmdir(path)
+static int NEAR realrmdir(path)
 CONST char *path;
 {
-	int n, dupdosdrive;
+	int n, flags;
 
-	dupdosdrive = dosdrive;
-	dosdrive = 0;
+	flags = pushswap();
 	n = Xrmdir(path);
-	dosdrive = dupdosdrive;
+	popswap(flags);
 
 	return(n);
 }
-#endif	/* !_NODOSDRIVE */
+#endif	/* DEP_PSEUDOPATH */
 
 VOID lostcwd(path)
 char *path;
@@ -888,15 +844,18 @@ char *path;
 	duperrno = errno;
 	if (!path) path = buf;
 
-	if (path != fullpath && nodoschdir(fullpath) >= 0 && nodosgetwd(path))
+	if (path != fullpath && realchdir(fullpath) >= 0 && realgetwd(path))
 		cp = NOCWD_K;
-	else if ((cp = gethomedir())
-	&& nodoschdir(cp) >= 0 && nodosgetwd(path))
+#ifdef	DEP_PSEUDOPATH
+	else if (unixpath && realchdir(unixpath) >= 0 && realgetwd(path))
+		cp = NOPSU_K;
+#endif
+	else if ((cp = gethomedir()) && realchdir(cp) >= 0 && realgetwd(path))
 		cp = GOHOM_K;
-	else if (nodoschdir(rootpath) >= 0 && nodosgetwd(path)) cp = GOROT_K;
+	else if (realchdir(rootpath) >= 0 && realgetwd(path)) cp = GOROT_K;
 	else error(rootpath);
 
-	if (path != fullpath) strncpy2(fullpath, path, MAXPATHLEN - 1);
+	if (path != fullpath) Xstrncpy(fullpath, path, MAXPATHLEN - 1);
 	warning(0, cp);
 #ifndef	_NOUSEHASH
 	searchhash(NULL, nullstr, nullstr);
@@ -912,6 +871,9 @@ CONST char *src, *dest;
 
 	if (Xstat(src, &st1) < 0 || Xstat(dest, &st2) < 0) return(0);
 	if (st1.st_dev != st2.st_dev || st1.st_ino != st2.st_ino) return(0);
+# ifdef	DEP_URLPATH
+	if (st1.st_dev == (dev_t)-1 || st1.st_ino == (ino_t)-1) return(0);
+# endif
 
 	return(1);
 }
@@ -952,7 +914,7 @@ struct stat *stp1, *stp2;
 #endif
 	char buf[BUFSIZ];
 	off_t total;
-	int n, fd1, fd2, tty, flags, mode, duperrno;
+	int n, fd1, fd2, tty, flags, mode, timeout, duperrno;
 
 #if	MSDOS
 	if (!stp2 && Xlstat(dest, &st) >= 0) stp2 = &st;
@@ -961,10 +923,18 @@ struct stat *stp1, *stp2;
 #endif
 #ifndef	NOSYMLINK
 	if (s_islnk(stp1)) {
+# ifdef	DEP_URLPATH
+		fd1 = urlpath(src, NULL, NULL, NULL);
+		fd2 = urlpath(dest, NULL, NULL, NULL);
+		if (fd1 != fd2) {
+			if (Xstat(src, stp1) < 0) return(-1);
+		}
+		else
+# endif
 		if (cpsymlink(src, dest) < 0) return(-1);
 		else return(0);
 	}
-#endif
+#endif	/* !NOSYMLINK */
 
 	flags = (O_BINARY | O_RDONLY);
 	mode = stp1 -> st_mode;
@@ -992,17 +962,29 @@ struct stat *stp1, *stp2;
 	}
 
 	tty = isatty(fd1);
+	timeout = 0;
+#ifdef	DEP_URLPATH
+	switch (chkopenfd(fd1)) {
+		case DEV_URL:
+		case DEV_HTTP:
+			timeout = urltimeout;
+			VOID_C urlfstat(fd1, stp1);
+			break;
+		default:
+			break;
+	}
+#endif
+
 	total = (off_t)0;
+	if (tty) stp1 -> st_size = (off_t)-1;
 	for (;;) {
-		if (!tty && total >= stp1 -> st_size) n = 0;
-		else n = sureread(fd1, buf, sizeof(buf));
+		if (stp1 -> st_size >= (off_t)0 && total >= stp1 -> st_size)
+			n = 0;
+		else n = checkread(fd1, buf, sizeof(buf), timeout);
 		if (n < 0) break;
 		total += (off_t)n;
 		if (!n) {
-			if (!tty && total < stp1 -> st_size) {
-				duperrno = ETIMEDOUT;
-				n = -1;
-			}
+			if (total < stp1 -> st_size) n = seterrno(ETIMEDOUT);
 			break;
 		}
 #ifndef	_NOEXTRACOPY
@@ -1044,7 +1026,7 @@ int safecpfile(src, dest, stp1, stp2)
 CONST char *src, *dest;
 struct stat *stp1, *stp2;
 {
-#ifndef	_NODOSDRIVE
+#ifdef	DEP_DOSDRIVE
 	int drive;
 #endif
 
@@ -1052,11 +1034,8 @@ struct stat *stp1, *stp2;
 		if (cpfile(src, dest, stp1, stp2) >= 0) break;
 		if (errno && errno != ENOSPC) return(-1);
 		for (;;) {
-			if (!yesno(NOSPC_K)) {
-				errno = ENOSPC;
-				return(-1);
-			}
-#ifndef	_NODOSDRIVE
+			if (!yesno(NOSPC_K)) return(seterrno(ENOSPC));
+#ifdef	DEP_DOSDRIVE
 			if ((drive = dospath3(dest))) {
 				if (flushdrv(drive, changemes) < 0) continue;
 			}
@@ -1096,7 +1075,7 @@ int max;
 	time_t now;
 
 	if (last < 0L) {
-		now = time2();
+		now = Xtime(NULL);
 		last = ((now & 0xff) << 16) + (now & ~0xff) + getpid();
 	}
 
@@ -1151,64 +1130,61 @@ char *dir;
 	char *cp, path[MAXPATHLEN];
 	int n, len, mask;
 
-	if (!deftmpdir || !*deftmpdir || !dir) {
-		errno = ENOENT;
-		return(-1);
-	}
-	realpath2(deftmpdir, path, 1);
-	free(deftmpdir);
-#if	MSDOS && !defined (_NOUSELFN) && !defined (_NODOSDRIVE)
-	if (checkdrive(toupper2(path[0]) - 'A') && !nodosgetwd(path)) {
+	if (!deftmpdir || !*deftmpdir || !dir) return(seterrno(ENOENT));
+	Xrealpath(deftmpdir, path, RLP_READLINK);
+	Xfree(deftmpdir);
+#if	defined (DEP_DOSDRIVE) && defined (DEP_DOSLFN)
+	if (checkdrive(Xtoupper(path[0]) - 'A') && !realgetwd(path)) {
 		lostcwd(path);
 		deftmpdir = NULL;
 		return(-1);
 	}
 #endif
-	deftmpdir = strdup2(path);
+	deftmpdir = Xstrdup(path);
 #if	MSDOS
-	*path = (isupper2(getcurdrv())) ? toupper2(*path) : tolower2(*path);
+	*path = (Xisupper(getcurdrv())) ? Xtoupper(*path) : Xtolower(*path);
 #endif
 
 	mask = 0777 & ~tmpumask;
 	cp = strcatdelim(path);
 	if (tmpfilename) {
-		strcpy(cp, tmpfilename);
+		Xstrcpy(cp, tmpfilename);
 		if (Xaccess(path, R_OK | W_OK | X_OK) < 0) {
-			free(tmpfilename);
+			Xfree(tmpfilename);
 			tmpfilename = NULL;
 		}
 	}
 	if (!tmpfilename) {
 		n = strsize(TMPPREFIX);
-		strncpy(cp, TMPPREFIX, n);
+		Xstrncpy(cp, TMPPREFIX, n);
 		len = strsize(path) - (cp - path);
 		if (len > MAXTMPNAMLEN) len = MAXTMPNAMLEN;
 		len -= n;
 		genrandname(NULL, 0);
 
 		for (;;) {
-			genrandname(cp + n, len);
-			if (nodosmkdir(path, mask) >= 0) break;
+			genrandname(&(cp[n]), len);
+			if (realmkdir(path, mask) >= 0) break;
 			if (errno != EEXIST) return(-1);
 		}
-		tmpfilename = strdup2(cp);
+		tmpfilename = Xstrdup(cp);
 	}
 
 	if (!(n = strlen(dir))) {
-		strcpy(dir, path);
+		Xstrcpy(dir, path);
 		return(0);
 	}
 
-	strncpy((cp = strcatdelim(path)), dir, n);
+	Xstrncpy((cp = strcatdelim(path)), dir, n);
 	len = strsize(path) - (cp - path);
 	if (len > MAXTMPNAMLEN) len = MAXTMPNAMLEN;
 	len -= n;
 	genrandname(NULL, 0);
 
 	for (;;) {
-		genrandname(cp + n, len);
-		if (nodosmkdir(path, mask) >= 0) {
-			strcpy(dir, path);
+		genrandname(&(cp[n]), len);
+		if (realmkdir(path, mask) >= 0) {
+			Xstrcpy(dir, path);
 			return(0);
 		}
 		if (errno != EEXIST) break;
@@ -1217,7 +1193,7 @@ char *dir;
 	if (cp > path) {
 		*(--cp) = '\0';
 		n = errno;
-		nodosrmdir(path);
+		realrmdir(path);
 		errno = n;
 	}
 
@@ -1229,14 +1205,12 @@ CONST char *dir;
 {
 	char path[MAXPATHLEN];
 
-	if (dir && *dir && nodosrmdir(dir) < 0) return(-1);
-	if (!deftmpdir || !*deftmpdir || !tmpfilename || !*tmpfilename) {
-		errno = ENOENT;
-		return(-1);
-	}
+	if (dir && *dir && realrmdir(dir) < 0) return(-1);
+	if (!deftmpdir || !*deftmpdir || !tmpfilename || !*tmpfilename)
+		return(seterrno(ENOENT));
 	strcatdelim2(path, deftmpdir, tmpfilename);
-	if (nodosrmdir(path) >= 0) {
-		free(tmpfilename);
+	if (realrmdir(path) >= 0) {
+		Xfree(tmpfilename);
 		tmpfilename = NULL;
 	}
 	else if (errno != ENOTEMPTY && errno != EEXIST && errno != EACCES)
@@ -1277,7 +1251,7 @@ char *file;
 	if (mktmpdir(path) < 0) return(-1);
 	VOID_C strcatdelim(path);
 	if ((fd = opentmpfile(path, 0666 & ~tmpumask)) >= 0) {
-		strcpy(file, path);
+		Xstrcpy(file, path);
 		return(fd);
 	}
 	duperrno = errno;
@@ -1317,7 +1291,7 @@ VOID removetmp(dir, file)
 char *dir;
 CONST char *file;
 {
-#ifdef	_USEDOSEMU
+#ifdef	DEP_DOSEMU
 	char path[MAXPATHLEN];
 #endif
 
@@ -1331,13 +1305,13 @@ CONST char *file;
 	else if (Xunlink(nodospath(path, file)) < 0) warning(-1, file);
 
 	if (_chdir2(fullpath) < 0) lostcwd(fullpath);
-#ifndef	_NODOSDRIVE
-	shutdrv(dosdrv);
-	dosdrv = -1;
+#ifdef	DEP_PSEUDOPATH
+	shutdrv(pseudodrv);
+	pseudodrv = -1;
 #endif
 	if (dir) {
 		if (*dir && rmtmpdir(dir) < 0) warning(-1, dir);
-		free(dir);
+		Xfree(dir);
 	}
 }
 
@@ -1348,21 +1322,32 @@ CONST char *dir, *file;
 	p_id_t pid;
 #endif
 	extern char **environ;
-	char buf[MAXPATHLEN];
+	char *argv[4], buf[MAXPATHLEN];
 
-	VOID_C excllock(NULL, LCK_UNLOCK);
+	VOID_C excllock(NULL, LOCK_UN);
 
 	if (!dir || !*dir || !file || !*file) return(0);
 	strcatdelim2(buf, dir, file);
 
-	if (rawchdir(buf) != 0) return(0);
-	rawchdir(rootpath);
+	if (rawchdir(buf) < 0) return(0);
+	VOID_C rawchdir(rootpath);
+
 #if	MSDOS
-	spawnlpe(P_WAIT, "DELTREE.EXE", "DELTREE", "/Y", buf, NULL, environ);
+	argv[0] = "DELTREE";
+	argv[1] = "/Y";
+#else
+	argv[0] = "rm";
+	argv[1] = "-rf";
+#endif
+	argv[2] = buf;
+	argv[3] = NULL;
+
+#if	MSDOS
+	spawnve(P_WAIT, "DELTREE.EXE", argv, environ);
 #else
 	if ((pid = fork()) < (p_id_t)0) return(-1);
 	else if (!pid) {
-		execle("/bin/rm", "rm", "-rf", buf, NULL, environ);
+		execve("/bin/rm", argv, environ);
 		_exit(1);
 	}
 #endif
@@ -1370,7 +1355,7 @@ CONST char *dir, *file;
 	return(0);
 }
 
-#ifndef	_NODOSDRIVE
+#ifdef	DEP_PSEUDOPATH
 char *dostmpdir(drive)
 int drive;
 {
@@ -1381,7 +1366,7 @@ int drive;
 	path[2] = '\0';
 	if (mktmpdir(path) < 0) return(NULL);
 
-	return(strdup2(path));
+	return(Xstrdup(path));
 }
 
 int tmpdosdupl(dir, dirp, single)
@@ -1389,25 +1374,33 @@ CONST char *dir;
 char **dirp;
 int single;
 {
-# ifdef	_USEDOSEMU
+# ifdef	DEP_DOSEMU
 	char tmp[MAXPATHLEN];
 # endif
 	struct stat st;
 	char *cp, *tmpdir, path[MAXPATHLEN];
 	int i, drive;
 
-	if (!(drive = dospath2(dir))) return(0);
+# ifdef	DEP_DOSDRIVE
+	if ((drive = dospath2(dir))) /*EMPTY*/;
+	else
+# endif
+# ifdef	DEP_URLPATH
+	if (urlpath(dir, NULL, NULL, &drive)) drive += '0';
+	else
+# endif
+	return(0);
 	if (!(tmpdir = dostmpdir(drive))) {
 		warning(-1, dir);
 		return(-1);
 	}
 
-	strcpy(path, tmpdir);
+	Xstrcpy(path, tmpdir);
 	cp = strcatdelim(path);
 	waitmes();
 
 	if (single || mark <= 0) {
-		strcpy(cp, filelist[filepos].name);
+		Xstrcpy(cp, filelist[filepos].name);
 		st.st_mode = filelist[filepos].st_mode;
 		st.st_atime = st.st_mtime = filelist[filepos].st_mtim;
 		st.st_size = filelist[filepos].st_size;
@@ -1419,7 +1412,7 @@ int single;
 	}
 	else for (i = 0; i < maxfile; i++) {
 		if (!ismark(&(filelist[i]))) continue;
-		strcpy(cp, filelist[i].name);
+		Xstrcpy(cp, filelist[i].name);
 		st.st_mode = filelist[i].st_mode;
 		st.st_atime = st.st_mtime = filelist[i].st_mtim;
 		st.st_size = filelist[i].st_size;
@@ -1430,7 +1423,7 @@ int single;
 		}
 	}
 
-	if (preparedrv(dospath(fullpath, NULL), &dosdrv) < 0) {
+	if ((pseudodrv = preparedrv(fullpath, NULL, NULL)) < 0) {
 		warning(-1, fullpath);
 		removetmp(tmpdir, NULL);
 		return(-1);
@@ -1451,12 +1444,16 @@ CONST char *file;
 {
 	struct stat st;
 	char path[MAXPATHLEN];
-# ifdef	_USEDOSEMU
+# ifdef	DEP_DOSEMU
 	char tmp[MAXPATHLEN];
 # endif
 
-	strcpy(gendospath(path, drive, '\0'), file);
-# ifdef	_USEDOSEMU
+# ifdef	DEP_DOSDRIVE
+	if (isalpha(drive)) Xstrcpy(gendospath(path, drive, '\0'), file);
+	else
+# endif
+	strcatdelim2(path, fullpath, file);
+# ifdef	DEP_DOSEMU
 	file = nodospath(tmp, file);
 # endif
 	waitmes();
@@ -1465,7 +1462,7 @@ CONST char *file;
 
 	return(0);
 }
-#endif	/* !_NODOSDRIVE */
+#endif	/* DEP_PSEUDOPATH */
 
 #ifndef	_NOWRITEFS
 static int NEAR isexist(file)
@@ -1497,13 +1494,13 @@ int fat, boundary, dirsize, ofs;
 				if (dot || !i || i > DOSBODYLEN) lfn = 1;
 				dot = i + 1;
 			}
-			else if (issjis1(s[i]) && issjis2(s[i + 1])) {
+			else if (iswsjis(s, i)) {
 				i++;
 				lfn = 1;
 			}
 			else if (lfn) /*EMPTY*/;
-			else if (islower2(s[i])) lfn = 1;
-			else if (strchr(LFNONLY, s[i])) lfn = 1;
+			else if (Xislower(s[i])) lfn = 1;
+			else if (Xstrchr(LFNONLY, s[i])) lfn = 1;
 		}
 		if (lfn) /*EMPTY*/;
 		else if (dot) {
@@ -1539,12 +1536,12 @@ int size, fat, boundary, dirsize, ofs;
 static int NEAR saferename(from, to)
 CONST char *from, *to;
 {
-#ifdef	_USEDOSEMU
+#ifdef	DEP_DOSEMU
 	char fpath[MAXPATHLEN], tpath[MAXPATHLEN];
 #endif
 
 	if (!strpathcmp2(from, to)) return(0);
-#ifdef	_USEDOSEMU
+#ifdef	DEP_DOSEMU
 	from = nodospath(fpath, from);
 	to = nodospath(tpath, to);
 #endif
@@ -1557,14 +1554,11 @@ static char *NEAR maketmpfile(len, fat, tmpdir, old)
 int len, fat;
 CONST char *tmpdir, *old;
 {
-#ifndef	PATHNOCASE
-	int i;
-#endif
 	char *fname, path[MAXPATHLEN];
 	int l, fd;
 
 	if (len < 0) return(NULL);
-	fname = malloc2(len + 1);
+	fname = Xmalloc(len + 1);
 	genrandname(NULL, 0);
 
 	if (tmpdir) l = strcatdelim2(path, tmpdir, NULL) - path;
@@ -1575,11 +1569,10 @@ CONST char *tmpdir, *old;
 	for (;;) {
 		genrandname(fname, len);
 #ifndef	PATHNOCASE
-		if (fat != FAT_NONE) for (i = 0; fname[i]; i++)
-			fname[i] = toupper2(fname[i]);
+		if (fat != FAT_NONE) Xstrtoupper(fname);
 #endif
 		if (tmpdir) {
-			strcpy(&(path[l]), fname);
+			Xstrcpy(&(path[l]), fname);
 			if (!isexist(path)) {
 				if (old) {
 #if	!MSDOS
@@ -1610,7 +1603,7 @@ CONST char *tmpdir, *old;
 			if (errno != EEXIST) break;
 		}
 	}
-	free(fname);
+	Xfree(fname);
 
 	return(NULL);
 }
@@ -1638,15 +1631,15 @@ off_t bsiz;
 	|| (fd = Xopen(dir, O_BINARY | O_RDONLY, 0666)) < 0)
 		return(NULL);
 	n = (off_t)(st.st_size) / bsiz;
-	tmp = (u_char *)malloc2(n + 1);
+	tmp = (u_char *)Xmalloc(n + 1);
 
 	for (i = 0; i < n; i++) {
 		if (Xlseek(fd, (off_t)(i * bsiz + 3), L_SET) < 0) {
-			free(tmp);
+			Xfree(tmp);
 			return(NULL);
 		}
 		if (sureread(fd, &ch, sizeof(ch)) < 0) {
-			free(tmp);
+			Xfree(tmp);
 			return(NULL);
 		}
 		tmp[i] = ch + 1;
@@ -1670,15 +1663,15 @@ int fnamp;
 		while ((dp = Xreaddir(dirp))) {
 			if (isdotdir(dp -> d_name)) continue;
 			else {
-				strcpy(&(path[fnamp]), dp -> d_name);
+				Xstrcpy(&(path[fnamp]), dp -> d_name);
 				if (saferename(path, dp -> d_name) < 0)
 					warning(-1, path);
 			}
 		}
-		Xclosedir(dirp);
+		VOID_C Xclosedir(dirp);
 	}
 	if (Xrmdir(dir) < 0) warning(-1, dir);
-	free(dir);
+	Xfree(dir);
 }
 
 VOID arrangedir(fs)
@@ -1723,7 +1716,7 @@ int fs;
 			dirsize = 4;	/* short + short */
 			namofs = 3;
 			break;
-# ifndef	_NODOSDRIVE
+# ifdef	DEP_DOSDRIVE
 		case FSID_DOSDRIVE:	/* Windows95 File System on DOSDRIVE */
 			fat = FAT_DOSDRIVE;
 			headbyte = -1;
@@ -1763,17 +1756,17 @@ int fs;
 	}
 
 	top = -1;
-	fnamelist = (char **)malloc2((maxfile + 1) * sizeof(char *));
+	fnamelist = (char **)Xmalloc((maxfile + 1) * sizeof(char *));
 	for (i = 0; i < maxfile; i++) {
 		if (isdotdir(filelist[i].name)) cp = filelist[i].name;
 		else {
 			if (top < 0) top = i;
 			cp = convput(path, filelist[i].name, 1, 0, NULL, NULL);
 		}
-#ifdef	_USEDOSEMU
+#ifdef	DEP_DOSEMU
 		if (_dospath(cp)) cp += 2;
 #endif
-		fnamelist[i] = strdup2(cp);
+		fnamelist[i] = Xstrdup(cp);
 	}
 	fnamelist[i] = NULL;
 	if (top < 0) {
@@ -1820,7 +1813,7 @@ int fs;
 #if	MSDOS
 			if (!(dp -> d_alias[0])) len = DOSBODYLEN;
 #else	/* !MSDOS */
-# ifndef	_NODOSDRIVE
+# ifdef	DEP_DOSDRIVE
 			if (fat == FAT_DOSDRIVE
 			&& wrap_reclen(dp) == DOSDIRENT)
 				len = DOSBODYLEN;
@@ -1829,9 +1822,9 @@ int fs;
 			ent = i;
 		}
 		else {
-			strcpy(&(path[fnamp]), dp -> d_name);
+			Xstrcpy(&(path[fnamp]), dp -> d_name);
 			if (saferename(dp -> d_name, path) < 0) {
-				Xclosedir(dirp);
+				VOID_C Xclosedir(dirp);
 				warning(-1, dp -> d_name);
 				restorefile(tmpdir, path, fnamp);
 				freevar(fnamelist);
@@ -1844,7 +1837,7 @@ int fs;
 		}
 		i++;
 	}
-	Xclosedir(dirp);
+	VOID_C Xclosedir(dirp);
 
 	if (ent > 0) {
 		if (!(tmp = maketmpfile(len, fat, tmpdir, tmpdir))) {
@@ -1857,7 +1850,7 @@ int fs;
 			noconv--;
 			return;
 		}
-		free(tmpdir);
+		Xfree(tmpdir);
 		tmpdir = tmp;
 		fnamp = strcatdelim2(path, tmpdir, NULL) - path;
 	}
@@ -1916,7 +1909,7 @@ int fs;
 			if (entnum) totalptr = entnum[++block];
 		}
 #endif	/* !MSDOS */
-		strcpy(&(path[fnamp]), fnamelist[i]);
+		Xstrcpy(&(path[fnamp]), fnamelist[i]);
 		if (saferename(path, fnamelist[i]) < 0) {
 			warning(-1, path);
 			break;
@@ -1927,24 +1920,24 @@ int fs;
 #endif
 	}
 #if	!MSDOS
-	if (entnum) free(entnum);
+	Xfree(entnum);
 	if (tmpfiles) {
 		for (i = 0; i < tmpno; i++) if (tmpfiles[i]) {
 			if (Xunlink(tmpfiles[i]) < 0) warning(-1, tmpfiles[i]);
-			free(tmpfiles[i]);
+			Xfree(tmpfiles[i]);
 		}
-		free(tmpfiles);
+		Xfree(tmpfiles);
 	}
 #endif
 
 	if (!(tmp = maketmpfile(len, fat, tmpdir, tmpdir)))
 		warning(-1, tmpdir);
 	else {
-		free(tmpdir);
+		Xfree(tmpdir);
 		tmpdir = tmp;
 	}
 	fnamp = strcatdelim2(path, tmpdir, NULL) - path;
-	snprintf2(&(path[fnamp]), sizeof(path) - fnamp, fnamelist[top]);
+	VOID_C Xsnprintf(&(path[fnamp]), sizeof(path) - fnamp, fnamelist[top]);
 	if (saferename(path, fnamelist[top]) < 0) warning(-1, path);
 	restorefile(tmpdir, path, fnamp);
 
