@@ -4,48 +4,66 @@
  *	information module
  */
 
-#include <errno.h>
 #include "fd.h"
+#include "dirent.h"
+#include "sysemu.h"
 #include "pathname.h"
+#include "term.h"
+#include "types.h"
 #include "funcno.h"
 #include "kanji.h"
 
-#ifdef	NOERRNO
-extern int errno;
+#ifdef	USEMNTENTH
+#include <mntent.h>
 #endif
-
-#ifdef	USESYSDIRH
-#include <sys/dir.h>
+#ifdef	USEMNTTABH
+#include <sys/mnttab.h>
 #endif
-
-#if	MSDOS
-#undef	USEVFSH
-#undef	USEMNTENTH
-#include "unixemu.h"
-#else
-#include <sys/param.h>
-#include <sys/file.h>
+#ifdef	USEMNTCTL
+#include <fshelp.h>
+#include <sys/vfs.h>
+#include <sys/mntctl.h>
+#include <sys/vmount.h>
+#endif
+#if	(!defined (USEMOUNTH) && !defined (USEFSDATA) \
+&& (defined (USEGETFSSTAT) || defined (USEGETVFSTAT) \
+|| defined (USEMNTINFOR) || defined (USEMNTINFO))) \
+|| defined (USEMOUNTH) || defined (USEFSDATA)
+#include <sys/mount.h>
+#endif
+#if	defined (USEGETFSSTAT) || defined (USEGETMNT)
+#include <sys/fs_types.h>
+#endif
+#ifdef	USEGETFSENT
+#include <fstab.h>
+#endif
+#if	defined (USESTATVFSH) || defined (USEGETVFSTAT)
+#include <sys/statvfs.h>
+#endif
+#ifdef	USESTATFSH
+#include <sys/statfs.h>
+#endif
+#ifdef	USEVFSH
+#include <sys/vfs.h>
 #endif
 
 #define	KEYWID			7
 
 #ifdef	USEMNTENTH
-#include <mntent.h>
 typedef struct mntent		mnt_t;
-#define	setmntent2		setmntent
-#define	getmntent2(f,m)		getmntent(f)
-#define	hasmntopt2(m,o)		strmntopt((m) -> mnt_opts, o)
-#define	endmntent2		endmntent
+#define	Xsetmntent		setmntent
+#define	Xgetmntent(f,m)		getmntent(f)
+#define	Xhasmntopt(m,o)		strmntopt((m) -> mnt_opts, o)
+#define	Xendmntent		endmntent
 #endif
 
 #ifdef	USEMNTTABH
-#include <sys/mnttab.h>
 #define	MOUNTED			MNTTAB
 typedef struct mnttab		mnt_t;
-#define	setmntent2		fopen
-#define	getmntent2(f,m)		(getmntent(f, m) ? NULL : m)
-#define	hasmntopt2(m,o)		strmntopt((m) -> mnt_mntopts, o)
-#define	endmntent2		fclose
+#define	Xsetmntent		fopen
+#define	Xgetmntent(f,m)		(getmntent(f, m) ? NULL : m)
+#define	Xhasmntopt(m,o)		strmntopt((m) -> mnt_mntopts, o)
+#define	Xendmntent		fclose
 #define	mnt_dir			mnt_mountp
 #define	mnt_fsname		mnt_special
 #define	mnt_type		mnt_fstype
@@ -54,52 +72,34 @@ typedef struct mnttab		mnt_t;
 # endif
 #endif	/* USEMNTTABH */
 
-#ifdef	USEMNTCTL
-#include <fshelp.h>
-#include <sys/vfs.h>
-#include <sys/mntctl.h>
-#include <sys/vmount.h>
-#endif
-
-#if	!defined (USEMOUNTH) && !defined (USEFSDATA) \
-&& (defined (USEGETFSSTAT) || defined (USEGETVFSTAT) \
-|| defined (USEMNTINFOR) || defined (USEMNTINFO))
-#include <sys/mount.h>
-#endif
-
-#if	defined (USEGETFSSTAT) || defined (USEGETMNT)
-#include <sys/fs_types.h>
-#endif
-
 #if	defined (USEGETFSSTAT) || defined (USEGETVFSTAT) \
 || defined (USEMNTCTL) || defined (USEMNTINFOR) || defined (USEMNTINFO) \
-|| defined (USEGETMNT)
+|| defined (USEGETMNT) || defined (MINIX)
 typedef struct _mnt_t {
 	CONST char *mnt_fsname;
 	CONST char *mnt_dir;
 	CONST char *mnt_type;
 	CONST char *mnt_opts;
 } mnt_t;
-static FILE *NEAR setmntent2 __P_((CONST char *, CONST char *));
-static mnt_t *NEAR getmntent2 __P_((FILE *, mnt_t *));
-#define	hasmntopt2(m,o)		strmntopt((m) -> mnt_opts, o)
-# if	defined (USEMNTINFO) || defined (USEGETMNT)
-# define	endmntent2(f)
-# else
-# define	endmntent2	free
-# endif
-static int mnt_ptr = 0;
-static int mnt_size = 0;
+#define	Xhasmntopt(m,o)		strmntopt((m) -> mnt_opts, o)
+# ifdef	MINIX
+# define	Xendmntent	fclose
+# else	/* !MINIX */
+#  if	defined (USEMNTINFO) || defined (USEGETMNT)
+#  define	Xendmntent(f)
+#  else
+#  define	Xendmntent	Xfree
+#  endif
+# endif	/* !MINIX */
 #endif	/* USEGETFSSTAT || USEGETVFSTAT || USEMNTCTL \
-|| USEMNTINFOR || USEMNTINFO || USEGETMNT */
+|| USEMNTINFOR || USEMNTINFO || USEGETMNT || MINIX */
 
 #ifdef	USEGETFSENT
-#include <fstab.h>
 typedef struct fstab		mnt_t;
-#define	setmntent2(f,m)		(FILE *)(setfsent(), NULL)
-#define	getmntent2(f,m)		getfsent()
-#define	hasmntopt2(m,o)		strmntopt((m) -> fs_mntops, o)
-#define	endmntent2(fp)		endfsent()
+#define	Xsetmntent(f,m)		(FILE *)(setfsent(), NULL)
+#define	Xgetmntent(f,m)		getfsent()
+#define	Xhasmntopt(m,o)		strmntopt((m) -> fs_mntops, o)
+#define	Xendmntent(fp)		endfsent()
 #define	mnt_dir			fs_file
 #define	mnt_fsname		fs_spec
 #define	mnt_type		fs_vfstype
@@ -117,7 +117,7 @@ typedef struct _mnt_t {
 	CONST char *mnt_type;
 	CONST char *mnt_opts;
 } mnt_t;
-#define	hasmntopt2(m,o)		strmntopt((m) -> mnt_opts, o)
+#define	Xhasmntopt(m,o)		strmntopt((m) -> mnt_opts, o)
 # ifdef	PC98
 # define	PT_FAT12	0x81	/* 0x80 | 0x01 */
 # define	PT_FAT16	0x91	/* 0x80 | 0x11 */
@@ -132,10 +132,6 @@ typedef struct _mnt_t {
 # endif
 #endif	/* MSDOS */
 
-#if	defined (USESTATVFSH) || defined (USEGETVFSTAT)
-#include <sys/statvfs.h>
-#endif
-
 #ifdef	USESTATVFSH
 # ifdef	USESTATVFS_T
 typedef statvfs_t		statfs_t;
@@ -147,26 +143,22 @@ typedef struct statvfs		statfs_t;
 #endif	/* USESTATVFSH */
 
 #ifdef	USESTATFSH
-#include <sys/statfs.h>
 #define	f_bavail		f_bfree
 typedef struct statfs		statfs_t;
 #define	blocksize(fs)		(fs).f_bsize
 #endif
 
 #ifdef	USEVFSH
-#include <sys/vfs.h>
 typedef struct statfs		statfs_t;
 #define	blocksize(fs)		(fs).f_bsize
 #endif
 
 #ifdef	USEMOUNTH
-#include <sys/mount.h>
 typedef struct statfs		statfs_t;
 #define	blocksize(fs)		(fs).f_bsize
 #endif
 
 #ifdef	USEFSDATA
-#include <sys/mount.h>
 typedef struct fs_data		statfs_t;
 #define	statfs2(p, b)		(statfs(p, b) - 1)
 #define	blocksize(fs)		1024
@@ -187,9 +179,13 @@ typedef struct fs_data		statfs_t;
 # else	/* STATFSARGS < 4 */
 #  if	(STATFSARGS == 3)
 #  define	statfs2(p, b)	statfs(p, b, sizeof(statfs_t))
-#  else
-#  define	statfs2		statfs
-#  endif
+#  else	/* STATFSARGS != 3 */
+#   ifdef	USEFSTATFS
+static int NEAR statfs2 __P_((CONST char *, statfs_t *));
+#   else
+#   define	statfs2		statfs
+#   endif
+#  endif	/* STATFSARGS != 3 */
 # endif	/* STATFSARGS < 4 */
 #endif	/* USESTATFSH || USEVFSH || USEMOUNTH */
 
@@ -221,60 +217,9 @@ extern int unixstatfs __P_((CONST char *, statfs_t *));
 # endif
 #endif	/* NETBSD */
 
-extern VOID error __P_((CONST char *));
-extern int _chdir2 __P_((CONST char *));
-extern char *strcpy2 __P_((char *, CONST char *));
-extern char *strncpy2 __P_((char *, CONST char *, int));
-extern char *getwd2 __P_((VOID_A));
-extern VOID warning __P_((int, CONST char *));
-#ifdef	_USEDOSPATH
-extern int dospath __P_((CONST char *, char *));
-extern char *gendospath __P_((char *, int, int));
-#endif
-#if	MSDOS && !defined (_NOUSELFN)
-extern int supportLFN __P_((CONST char *));
-# ifndef	_NODOSDRIVE
-extern int checkdrive __P_((int));
-# endif
-#endif	/* MSDOS && !_NOUSELFN */
-#ifndef	_NODOSDRIVE
-extern int dosstatfs __P_((int, char *));
-#endif
-extern char *malloc2 __P_((ALLOC_T));
-extern char *realloc2 __P_((VOID_P, ALLOC_T));
-#if	!MSDOS || !defined (NOFLOCK)
-extern int Xstat __P_((CONST char *, struct stat *));
-#endif
-extern int Xaccess __P_((CONST char *, int));
-extern int filetop __P_((int));
-extern VOID cputspace __P_((int));
-extern VOID cputstr __P_((int, CONST char *));
-#ifdef	_NOPTY
-#define	Xlocate			locate
-#define	Xputterm		putterm
-#define	Xputch2			putch2
-#define	Xcputs2			cputs2
-#define	Xcprintf2		cprintf2
-#else
-extern VOID Xlocate __P_((int, int));
-extern VOID Xputterm __P_((int));
-extern VOID Xputch2 __P_((int));
-extern VOID Xcputs2 __P_((CONST char *));
-extern VOID Xcprintf2 __P_((CONST char *, ...));
-#endif
-
-extern bindtable bindlist[];
-extern CONST functable funclist[];
-extern char fullpath[];
-extern char *distributor;
-#ifndef	_NODOSDRIVE
-extern int needbavail;
-#endif
-
 #ifndef	MOUNTED
 #define	MOUNTED			"/etc/mtab"
 #endif
-
 #ifndef	MNTTYPE_43
 #define	MNTTYPE_43		"4.3"		/* NEWS-OS 3-4, HP-UX, HI-UX */
 #endif
@@ -349,9 +294,56 @@ extern int needbavail;
 #endif
 #define	MNTTYPE_XNFS		"nfs"		/* NFS */
 
+extern VOID error __P_((CONST char *));
+extern int _chdir2 __P_((CONST char *));
+extern VOID warning __P_((int, CONST char *));
+#ifdef	DEP_DOSLFN
+extern int supportLFN __P_((CONST char *));
+# ifdef	DEP_DOSDRIVE
+extern int checkdrive __P_((int));
+# endif
+#endif	/* DEP_DOSLFN */
+#ifdef	DEP_DOSDRIVE
+extern int dosstatfs __P_((int, char *));
+#endif
+extern int filetop __P_((int));
+extern VOID cputspace __P_((int));
+extern VOID cputstr __P_((int, CONST char *));
+#ifdef	DEP_PTY
+extern VOID Xlocate __P_((int, int));
+extern VOID Xputterm __P_((int));
+extern VOID XXputch __P_((int));
+extern VOID XXcputs __P_((CONST char *));
+extern int XXcprintf __P_((CONST char *, ...));
+#else
+#define	Xlocate			locate
+#define	Xputterm		putterm
+#define	XXputch			Xputch
+#define	XXcputs			Xcputs
+#define	XXcprintf		Xcprintf
+#endif
+
+extern bindlist_t bindlist;
+extern int maxbind;
+extern CONST functable funclist[];
+extern char fullpath[];
+extern char *distributor;
+#ifdef	DEP_DOSDRIVE
+extern int needbavail;
+#endif
+
 static int NEAR code2str __P_((char *, int));
 static int NEAR checkline __P_((int));
 VOID help __P_((int));
+#if	defined (USEGETFSSTAT) || defined (USEGETVFSTAT) \
+|| defined (USEMNTCTL) || defined (USEMNTINFOR) || defined (USEMNTINFO) \
+|| defined (USEGETMNT) || MINIX
+static FILE *NEAR Xsetmntent __P_((CONST char *, CONST char *));
+static mnt_t *NEAR Xgetmntent __P_((FILE *, mnt_t *));
+#endif
+#ifdef	MINIX
+static char *NEAR getmntfield __P_((char **));
+#endif
 static int NEAR getfsinfo __P_((CONST char *, statfs_t *, mnt_t *));
 static CONST char *NEAR strmntopt __P_((CONST char *, CONST char *));
 #ifndef	NOFLOCK
@@ -382,7 +374,7 @@ static CONST strtable mntlist[] = {
 	{FSID_LFN, MNTTYPE_FAT32},
 	{FSID_LFN, MNTTYPE_SHARED},
 #if	MSDOS
-# ifndef	_NODOSDRIVE
+# ifdef	DEP_DOSDRIVE
 	{FSID_FAT, MNTTYPE_FAT12},
 	{FSID_FAT, MNTTYPE_FAT16},
 	{FSID_FAT, MNTTYPE_FAT16X},
@@ -415,6 +407,12 @@ static CONST strtable mntlist[] = {
 #endif	/* !MSDOS */
 };
 #define	MNTLISTSIZ		arraysize(mntlist)
+#if	defined (USEGETFSSTAT) || defined (USEGETVFSTAT) \
+|| defined (USEMNTCTL) || defined (USEMNTINFOR) || defined (USEMNTINFO) \
+|| defined (USEGETMNT)
+static int mnt_ptr = 0;
+static int mnt_size = 0;
+#endif
 
 
 static int NEAR code2str(buf, code)
@@ -425,38 +423,42 @@ int code;
 
 	buf = buf + strlen(buf);
 	if (code >= K_F(1) && code <= K_F(20))
-		snprintf2(buf, KEYWID + 1, "F%-6d", code - K_F0);
-	else if ((code & ~0x7f) == 0x80 && isalpha2(code & 0x7f))
-		snprintf2(buf, KEYWID + 1, "Alt-%c  ", code & 0x7f);
+		VOID_C Xsnprintf(buf, KEYWID + 1, "F%-6d", code - K_F0);
+	else if ((code & ~0x7f) == 0x80 && Xisalpha(code & 0x7f))
+		VOID_C Xsnprintf(buf, KEYWID + 1, "Alt-%c  ", code & 0x7f);
 	else if (code == K_UP)
-		snprintf2(buf, KEYWID + 1, "%-*.*s", KEYWID, KEYWID, UPAR_K);
+		VOID_C Xsnprintf(buf, KEYWID + 1,
+			"%-*.*s", KEYWID, KEYWID, UPAR_K);
 	else if (code == K_DOWN)
-		snprintf2(buf, KEYWID + 1, "%-*.*s", KEYWID, KEYWID, DWNAR_K);
+		VOID_C Xsnprintf(buf, KEYWID + 1,
+			"%-*.*s", KEYWID, KEYWID, DWNAR_K);
 	else if (code == K_RIGHT)
-		snprintf2(buf, KEYWID + 1, "%-*.*s", KEYWID, KEYWID, RIGAR_K);
+		VOID_C Xsnprintf(buf, KEYWID + 1,
+			"%-*.*s", KEYWID, KEYWID, RIGAR_K);
 	else if (code == K_LEFT)
-		snprintf2(buf, KEYWID + 1, "%-*.*s", KEYWID, KEYWID, LEFAR_K);
+		VOID_C Xsnprintf(buf, KEYWID + 1,
+			"%-*.*s", KEYWID, KEYWID, LEFAR_K);
 	else {
 		for (i = 0; i < KEYCODESIZ; i++)
 			if (code == keycodelist[i]) break;
 		if (i < KEYCODESIZ)
-			snprintf2(buf, KEYWID + 1, "%-*.*s",
-				KEYWID, KEYWID, keystrlist[i]);
+			VOID_C Xsnprintf(buf, KEYWID + 1,
+				"%-*.*s", KEYWID, KEYWID, keystrlist[i]);
 #ifdef	CODEEUC
 		else if (isekana2(code))
-			snprintf2(buf, KEYWID + 1 + 1,
+			VOID_C Xsnprintf(buf, KEYWID + 1 + 1,
 				"'%c%c'    ", C_EKANA, code & 0xff);
 #endif
 		else if (code > MAXUTYPE(u_char)) return(0);
 #ifndef	CODEEUC
-		else if (iskana2(code))
-			snprintf2(buf, KEYWID + 1, "'%c'    ", code);
+		else if (Xiskana(code))
+			VOID_C Xsnprintf(buf, KEYWID + 1, "'%c'    ", code);
 #endif
-		else if (iscntrl2(code))
-			snprintf2(buf, KEYWID + 1,
+		else if (Xiscntrl(code))
+			VOID_C Xsnprintf(buf, KEYWID + 1,
 				"Ctrl-%c ", (code + '@') & 0x7f);
 		else if (!ismsb(code))
-			snprintf2(buf, KEYWID + 1, "'%c'    ", code);
+			VOID_C Xsnprintf(buf, KEYWID + 1, "'%c'    ", code);
 		else return(0);
 	}
 
@@ -477,17 +479,18 @@ int y;
 VOID help(arch)
 int arch;
 {
+	CONST char *cp;
 	char buf[(KEYWID + 1 + 1) * 2 + 1];
 	int i, j, c, x, y, yy;
 
 	if (distributor && *distributor) {
 		i = n_column - (int)strlen(distributor) - 24;
 		Xlocate(i, L_HELP);
-		Xputch2('[');
+		VOID_C XXputch('[');
 		Xputterm(T_STANDOUT);
-		Xcprintf2(" Distributed by: %s ", distributor);
+		VOID_C XXcprintf(" Distributed by: %s ", distributor);
 		Xputterm(END_STANDOUT);
-		Xputch2(']');
+		VOID_C XXputch(']');
 	}
 
 	yy = filetop(win);
@@ -504,23 +507,23 @@ int arch;
 
 		c = 0;
 		buf[0] = '\0';
-		for (j = 0; j < MAXBINDTABLE && bindlist[j].key >= 0; j++) {
+		for (j = 0; j < maxbind; j++) {
 			if (i != ffunc(j)) continue;
 			c += code2str(buf, (int)(bindlist[j].key));
 			if (c >= 2) break;
 		}
-		if (c < 2)
-		for (j = 0; j < MAXBINDTABLE && bindlist[j].key >= 0; j++) {
+		if (c < 2) for (j = 0; j < maxbind; j++) {
 			if (i != dfunc(j)) continue;
 			c += code2str(buf, (int)(bindlist[j].key));
 			if (c >= 2) break;
 		}
 		if (!c) continue;
 
-		Xcputs2("  ");
+		XXcputs("  ");
 		if (c < 2) cputspace(KEYWID);
-		Xcprintf2("%k: %k",
-			buf, mesconv(funclist[i].hmes, funclist[i].hmes_eng));
+		cp = mesconv2(funclist[i].hmes_no,
+			funclist[i].hmes, funclist[i].hmes_eng);
+		VOID_C XXcprintf("%k: %k", buf, cp);
 
 		y = checkline(y);
 	}
@@ -536,20 +539,20 @@ int arch;
 
 #ifdef	USEMNTCTL
 /*ARGSUSED*/
-static FILE *NEAR setmntent2(file, mode)
+static FILE *NEAR Xsetmntent(file, mode)
 CONST char *file, *mode;
 {
 	char *buf;
 
 	mntctl(MCTL_QUERY, sizeof(int), (struct vmount *)&mnt_size);
-	buf = malloc2(mnt_size);
+	buf = Xmalloc(mnt_size);
 	mntctl(MCTL_QUERY, mnt_size, (struct vmount *)buf);
 	mnt_ptr = 0;
 
 	return((FILE *)buf);
 }
 
-static mnt_t *NEAR getmntent2(fp, mntp)
+static mnt_t *NEAR Xgetmntent(fp, mntp)
 FILE *fp;
 mnt_t *mntp;
 {
@@ -568,31 +571,31 @@ mnt_t *mntp;
 	cp = &(buf[mnt_ptr + vmntp -> vmt_data[VMT_OBJECT].vmt_off]);
 	len = strlen(cp) + 1;
 	if (!(vmntp -> vmt_flags & MNT_REMOTE)) {
-		fsname = realloc2(fsname, len);
+		fsname = Xrealloc(fsname, len);
 		memcpy(fsname, cp, len);
 	}
 	else {
 		host = &(buf[mnt_ptr
 			+ vmntp -> vmt_data[VMT_HOSTNAME].vmt_off]);
 		len += strlen(host) + 1;
-		fsname = realloc2(fsname, len);
-		strcpy(strcpy2(strcpy2(fsname, host), ":"), cp);
+		fsname = Xrealloc(fsname, len);
+		Xstrcpy(Xstrcpy(Xstrcpy(fsname, host), ":"), cp);
 	}
 
 	cp = &(buf[mnt_ptr + vmntp -> vmt_data[VMT_STUB].vmt_off]);
 	len = strlen(cp) + 1;
-	dir = realloc2(dir, len);
+	dir = Xrealloc(dir, len);
 	memcpy(dir, cp, len);
 
 	entp = getvfsbytype(vmntp -> vmt_gfstype);
 	if (entp) {
 		cp = entp -> vfsent_name;
 		len = strlen(cp) + 1;
-		type = realloc2(type, len);
+		type = Xrealloc(type, len);
 		memcpy(type, cp, len);
 	}
 	else if (type) {
-		free(type);
+		Xfree(type);
 		type = NULL;
 	}
 
@@ -610,21 +613,21 @@ mnt_t *mntp;
 #if	defined (USEMNTINFOR) || defined (USEMNTINFO) \
 || defined (USEGETFSSTAT) || defined (USEGETVFSTAT)
 
-#ifdef	USEGETVFSTAT
-#define	f_flags			f_flag
-#define	getfsstat2		getvfsstat
+# ifdef	USEGETVFSTAT
+# define	f_flags			f_flag
+# define	getfsstat2		getvfsstat
 typedef struct statvfs		mntinfo_t;
-#else
-#define	getfsstat2		getfsstat
+# else
+# define	getfsstat2		getfsstat
 typedef struct statfs		mntinfo_t;
-#endif
+# endif
 
-#if	!defined (MNT_RDONLY) && defined (M_RDONLY)
-#define	MNT_RDONLY		M_RDONLY
-#endif
+# if	!defined (MNT_RDONLY) && defined (M_RDONLY)
+# define	MNT_RDONLY		M_RDONLY
+# endif
 
 /*ARGSUSED*/
-static FILE *NEAR setmntent2(file, mode)
+static FILE *NEAR Xsetmntent(file, mode)
 CONST char *file, *mode;
 {
 # ifndef	USEMNTINFO
@@ -647,7 +650,7 @@ CONST char *file, *mode;
 #  else
 	size = (getfsstat2(NULL, 0, MNT_WAIT) + 1) * sizeof(mntinfo_t);
 	if (size > 0) {
-		buf = (mntinfo_t *)malloc2(size);
+		buf = (mntinfo_t *)Xmalloc(size);
 		mnt_size = getfsstat2(buf, size, MNT_WAIT);
 	}
 #  endif
@@ -663,7 +666,7 @@ CONST char *file, *mode;
 	return((FILE *)buf);
 }
 
-static mnt_t *NEAR getmntent2(fp, mntp)
+static mnt_t *NEAR Xgetmntent(fp, mntp)
 FILE *fp;
 mnt_t *mntp;
 {
@@ -710,21 +713,21 @@ mnt_t *mntp;
 	_mtrace_file = "getmntent(start)";
 # endif
 	len = strlen(buf[mnt_ptr].f_mntfromname) + 1;
-	fsname = realloc2(fsname, len);
+	fsname = Xrealloc(fsname, len);
 	memcpy(fsname, buf[mnt_ptr].f_mntfromname, len);
 
 	len = strlen(buf[mnt_ptr].f_mntonname) + 1;
-	dir = realloc2(dir, len);
+	dir = Xrealloc(dir, len);
 	memcpy(dir, buf[mnt_ptr].f_mntonname, len);
 
 	cp = (char *)getvfsbynumber(buf[mnt_ptr].f_type);
 	if (cp) {
 		len = strlen(cp) + 1;
-		type = realloc2(type, len);
+		type = Xrealloc(type, len);
 		memcpy(type, cp, len);
 	}
 	else if (type) {
-		free(type);
+		Xfree(type);
 		type = NULL;
 	}
 # ifdef	DEBUG
@@ -748,7 +751,7 @@ mnt_t *mntp;
 
 #ifdef	USEGETMNT
 /*ARGSUSED*/
-static FILE *NEAR setmntent2(file, mode)
+static FILE *NEAR Xsetmntent(file, mode)
 CONST char *file, *mode;
 {
 	mnt_ptr = 0;
@@ -757,7 +760,7 @@ CONST char *file, *mode;
 }
 
 /*ARGSUSED*/
-static mnt_t *NEAR getmntent2(fp, mntp)
+static mnt_t *NEAR Xgetmntent(fp, mntp)
 FILE *fp;
 mnt_t *mntp;
 {
@@ -771,15 +774,15 @@ mnt_t *mntp;
 		return(NULL);
 
 	len = strlen(buf.fd_req.devname) + 1;
-	fsname = realloc2(fsname, len);
+	fsname = Xrealloc(fsname, len);
 	memcpy(fsname, buf.fd_req.devname, len);
 
 	len = strlen(buf.fd_req.path) + 1;
-	dir = realloc2(dir, len);
+	dir = Xrealloc(dir, len);
 	memcpy(dir, buf.fd_req.path, len);
 
 	len = strlen(gt_names[buf.fd_req.fstype]) + 1;
-	type = realloc2(type, len);
+	type = Xrealloc(type, len);
 	memcpy(type, gt_names[buf.fd_req.fstype], len);
 
 	mntp -> mnt_fsname = fsname;
@@ -791,19 +794,79 @@ mnt_t *mntp;
 }
 #endif	/* USEGETMNT */
 
+#ifdef	MINIX
+static FILE *NEAR Xsetmntent(file, mode)
+CONST char *file, *mode;
+{
+	return(fopen(file, mode));
+}
+
+static char *NEAR getmntfield(cpp)
+char **cpp;
+{
+	char *s;
+
+	if (!cpp || !*cpp || !**cpp) return(vnullstr);
+
+	while (**cpp && !Xisprint(**cpp)) (*cpp)++;
+	if (!**cpp) return(vnullstr);
+	s = *cpp;
+
+	while (**cpp && Xisprint(**cpp)) (*cpp)++;
+	if (**cpp) *((*cpp)++) = '\0';
+
+	return(s);
+}
+
+static mnt_t *NEAR Xgetmntent(fp, mntp)
+FILE *fp;
+mnt_t *mntp;
+{
+	static char buf[BUFSIZ];
+	char *cp;
+
+	if (!(cp = fgets(buf, sizeof(buf), fp))) return(NULL);
+
+	mntp -> mnt_fsname = mntp -> mnt_dir =
+	mntp -> mnt_type = mntp -> mnt_opts = NULL;
+
+	mntp -> mnt_fsname = getmntfield(&cp);
+	mntp -> mnt_dir = getmntfield(&cp);
+	mntp -> mnt_type = getmntfield(&cp);
+	mntp -> mnt_opts = getmntfield(&cp);
+
+	return(mntp);
+}
+#endif	/* MINIX */
+
+#ifdef	USEFSTATFS
+static int NEAR statfs2(path, buf)
+CONST char *path;
+statfs_t *buf;
+{
+	int n, fd;
+
+	if ((fd = Xopen(path, O_RDONLY, 0666)) < 0) return(-1);
+	n = fstatfs(fd, buf);
+	Xclose(fd);
+
+	return(n);
+}
+#endif	/* USEFSTATFS */
+
 static int NEAR getfsinfo(path, fsbuf, mntbuf)
 CONST char *path;
 statfs_t *fsbuf;
 mnt_t *mntbuf;
 {
 #if	MSDOS
-# ifndef	_NODOSDRIVE
+# ifdef	DEP_DOSDRIVE
 	int i;
 # endif
 	char buf[MAXPATHLEN];
 	int drive;
 #else	/* !MSDOS */
-# ifndef	_NODOSDRIVE
+# ifdef	DEP_DOSDRIVE
 	static char dosmntdir[4];
 	char buf[3 * sizeof(long) + 1];
 	int drv;
@@ -836,15 +899,13 @@ mnt_t *mntbuf;
 				cp += strlen(cp);
 			len = cp - path;
 		}
-		strncpy2(mntbuf -> mnt_dir, path, len);
+		Xstrncpy(mntbuf -> mnt_dir, path, len);
 	}
 	else
 # endif
 	VOID_C gendospath(mntbuf -> mnt_dir, drive, _SC_);
 
-# ifdef	_NOUSELFN
-	mntbuf -> mnt_type = MNTTYPE_PC;
-# else	/* !_NOUSELFN */
+# ifdef	DEP_DOSLFN
 	switch (supportLFN(mntbuf -> mnt_dir)) {
 		case 3:
 			mntbuf -> mnt_type = MNTTYPE_SHARED;
@@ -855,14 +916,14 @@ mnt_t *mntbuf;
 		case 1:
 			mntbuf -> mnt_type = MNTTYPE_DOS7;
 			break;
-#  ifndef	_NODOSDRIVE
+#  ifdef	DEP_DOSDRIVE
 		case -1:
 			mntbuf -> mnt_fsname = "LFN emurate";
 			mntbuf -> mnt_type = MNTTYPE_DOS7;
 			break;
 		case -2:
 			mntbuf -> mnt_fsname = "BIOS raw";
-			i = checkdrive(toupper2(mntbuf -> mnt_dir[0]) - 'A');
+			i = checkdrive(Xtoupper(mntbuf -> mnt_dir[0]) - 'A');
 			if (i == PT_FAT12) mntbuf -> mnt_type = MNTTYPE_FAT12;
 			else if (i == PT_FAT16)
 				mntbuf -> mnt_type = MNTTYPE_FAT16;
@@ -872,15 +933,16 @@ mnt_t *mntbuf;
 			break;
 #  endif
 		case -4:
-			errno = ENOENT;
-			return(-1);
+			return(seterrno(ENOENT));
 /*NOTREACHED*/
 			break;
 		default:
 			mntbuf -> mnt_type = MNTTYPE_PC;
 			break;
 	}
-# endif	/* !_NOUSELFN */
+# else	/* !DEP_DOSLFN */
+	mntbuf -> mnt_type = MNTTYPE_PC;
+# endif	/* !DEP_DOSLFN */
 	mntbuf -> mnt_opts = nullstr;
 
 	if (statfs2(mntbuf -> mnt_dir, fsbuf) < 0) return(-1);
@@ -893,14 +955,14 @@ mnt_t *mntbuf;
 		cp = getrealpath(path, dir, NULL);
 		if (cp != dir && cp != path) return(-1);
 	}
-# ifndef	_NODOSDRIVE
+# ifdef	DEP_DOSDRIVE
 	else if ((drv = dospath(path, NULL))) {
 		mntbuf -> mnt_fsname = vnullstr;
 		mntbuf -> mnt_dir = dosmntdir;
 		dosmntdir[0] = drv;
-		strcpy(&(dosmntdir[1]), ":\\");
+		Xstrcpy(&(dosmntdir[1]), ":\\");
 		mntbuf -> mnt_type =
-			(islower2(drv)) ? MNTTYPE_DOS7 : MNTTYPE_PC;
+			(Xislower(drv)) ? MNTTYPE_DOS7 : MNTTYPE_PC;
 		mntbuf -> mnt_opts = vnullstr;
 		if (dosstatfs(drv, buf) < 0) return(-1);
 		if (buf[3 * sizeof(long)] & 001)
@@ -916,25 +978,31 @@ mnt_t *mntbuf;
 #   ifdef	USESTATVFSH
 		fsbuf -> f_frsize = 0L;
 #   endif
+#   ifndef	NOFBLOCKS
 		fsbuf -> f_blocks = *((long *)&(buf[1 * sizeof(long)]));
+#   endif
+#   ifndef	NOFBFREE
 		fsbuf -> f_bfree =
 		fsbuf -> f_bavail = *((long *)&(buf[2 * sizeof(long)]));
+#   endif
 #  endif	/* !USEFSDATA */
+#  ifndef	NOFFILES
 		fsbuf -> f_files = -1L;
+#  endif
 
 		return(0);
 	}
-# endif	/* !_NODOSDRIVE */
+# endif	/* DEP_DOSDRIVE */
 	else if ((cp = getrealpath(path, dir, NULL)) != dir) return(-1);
 
 	if (cp == dir) {
 		match = (ALLOC_T)0;
 
-		if (!(fp = setmntent2(MOUNTED, "rb"))) return(-1);
+		if (!(fp = Xsetmntent(MOUNTED, "rb"))) return(-1);
 		for (;;) {
 # ifdef	DEBUG
 			_mtrace_file = "getmntent(start)";
-			mntp = getmntent2(fp, &mnt);
+			mntp = Xgetmntent(fp, &mnt);
 			if (_mtrace_file) _mtrace_file = NULL;
 			else {
 				_mtrace_file = "getmntent(end)";
@@ -942,7 +1010,7 @@ mnt_t *mntbuf;
 			}
 			if (!mntp) break;
 # else
-			if (!(mntp = getmntent2(fp, &mnt))) break;
+			if (!(mntp = Xgetmntent(fp, &mnt))) break;
 # endif
 			len = strlen(mntp -> mnt_dir);
 			if (len < match || strncmp(mntp -> mnt_dir, dir, len)
@@ -950,25 +1018,19 @@ mnt_t *mntbuf;
 			&& dir[len] && dir[len] != _SC_))
 				continue;
 			match = len;
-			strcpy(fsname, mntp -> mnt_fsname);
+			Xstrcpy(fsname, mntp -> mnt_fsname);
 		}
-		endmntent2(fp);
+		Xendmntent(fp);
 
-		if (!match) {
-			errno = ENOENT;
-			return(-1);
-		}
+		if (!match) return(seterrno(ENOENT));
 		cp = fsname;
 	}
 
-	if (!(fp = setmntent2(MOUNTED, "rb"))) return(-1);
-	while ((mntp = getmntent2(fp, &mnt)))
+	if (!(fp = Xsetmntent(MOUNTED, "rb"))) return(-1);
+	while ((mntp = Xgetmntent(fp, &mnt)))
 		if (!strcmp(cp, mntp -> mnt_fsname)) break;
-	endmntent2(fp);
-	if (!mntp) {
-		errno = ENOENT;
-		return(-1);
-	}
+	Xendmntent(fp);
+	if (!mntp) return(seterrno(ENOENT));
 	memcpy((char *)mntbuf, (char *)mntp, sizeof(mnt_t));
 
 	if (statfs2(mntbuf -> mnt_dir, fsbuf) >= 0) /*EMPTY*/;
@@ -988,7 +1050,7 @@ CONST char *s1, *s2;
 	for (cp = s1; cp && *cp;) {
 		if (!strncmp(cp, s2, len) && (!cp[len] || cp[len] == ','))
 			return(cp);
-		if ((cp = strchr(cp, ','))) cp++;
+		if ((cp = Xstrchr(cp, ','))) cp++;
 	}
 
 	return(NULL);
@@ -1005,11 +1067,11 @@ CONST char *path;
 	if (Xstat(path, &st) < 0) return(-1);
 	if (!s_isdir(&st)) {
 		if (!(cp = strrdelim(path, 1))) return(-1);
-		strncpy2(buf, path, cp - path);
+		Xstrncpy(buf, path, cp - path);
 		path = buf;
 	}
 	if (getfsinfo(path, NULL, &mntbuf) < 0) return(-1);
-	if (strncasecmp2(mntbuf.mnt_type, MNTTYPE_XNFS, strsize(MNTTYPE_XNFS)))
+	if (Xstrncasecmp(mntbuf.mnt_type, MNTTYPE_XNFS, strsize(MNTTYPE_XNFS)))
 		return(0);
 
 	return(1);
@@ -1019,19 +1081,19 @@ CONST char *path;
 int writablefs(path)
 CONST char *path;
 {
-#ifdef	_USEDOSEMU
+#ifdef	DEP_DOSEMU
 	int drv;
 #endif
 	mnt_t mntbuf;
 	int i;
 
 	if (Xaccess(path, R_OK | W_OK | X_OK) < 0) return(-1);
-#ifdef	_USEDOSEMU
+#ifdef	DEP_DOSEMU
 	if ((drv = dospath(path, NULL)))
-		return((isupper2(drv)) ? FSID_FAT : FSID_DOSDRIVE);
+		return((Xisupper(drv)) ? FSID_FAT : FSID_DOSDRIVE);
 #endif
 	if (getfsinfo(path, NULL, &mntbuf) < 0) return(0);
-	if (hasmntopt2(&mntbuf, "ro")) return(0);
+	if (Xhasmntopt(&mntbuf, "ro")) return(0);
 
 	for (i = 0; i < MNTLISTSIZ; i++)
 		if (!strcmp(mntbuf.mnt_type, mntlist[i].str))
@@ -1065,14 +1127,12 @@ CONST char *dir;
 		if (getfsinfo(dir, &fsbuf, NULL) >= 0)
 # endif
 			return((off_t)blocksize(fsbuf));
+# ifndef	NOSTBLKSIZE
 		if (Xstat(dir, &st) >= 0) return((off_t)(st.st_blksize));
+# endif
 	}
 
-# ifdef	DEV_BSIZE
 	return(DEV_BSIZE);
-# else
-	return((off_t)512);
-# endif
 #endif	/* !MSDOS */
 }
 
@@ -1087,13 +1147,13 @@ CONST char *s, *unit;
 	Xlocate(0, yy + y);
 	Xputterm(L_CLEAR);
 	Xlocate(n_column / 2 - 20, yy + y);
-	Xcprintf2("%-20.20k", ind);
+	VOID_C XXcprintf("%-20.20k", ind);
 	Xlocate(n_column / 2 + 2, yy + y);
 	if (s) {
 		width = n_column - (n_column / 2 + 2);
 		cputstr(width, s);
 	}
-	else Xcprintf2("%<'*qd %k", MAXCOLSCOMMA(3), n, unit);
+	else VOID_C XXcprintf("%<'*qd %k", MAXCOLSCOMMA(3), n, unit);
 
 	return(checkline(++y));
 }
@@ -1120,11 +1180,11 @@ off_t *totalp, *freep, *bsizep;
 	statfs_t fsbuf;
 	int n;
 
-#ifndef	_NODOSDRIVE
+#ifdef	DEP_DOSDRIVE
 	needbavail++;
 #endif
 	n = getfsinfo(path, &fsbuf, NULL);
-#ifndef	_NODOSDRIVE
+#ifdef	DEP_DOSDRIVE
 	needbavail--;
 #endif
 	if (n < 0) {
@@ -1132,8 +1192,16 @@ off_t *totalp, *freep, *bsizep;
 		return(-1);
 	}
 
+#ifdef	NOFBLOCKS
+	*totalp = (off_t)0;
+#else
 	*totalp = fsbuf.f_blocks;
+#endif
+#ifdef	NOFBFREE
+	*freep = (off_t)0;
+#else
 	*freep = fsbuf.f_bavail;
+#endif
 	*bsizep = blocksize(fsbuf);
 
 	return(0);
@@ -1146,12 +1214,12 @@ CONST char *path;
 	mnt_t mntbuf;
 	int y, yy;
 
-#ifndef	_NODOSDRIVE
+#ifdef	DEP_DOSDRIVE
 	needbavail++;
 #endif
 	if (getfsinfo(path, &fsbuf, &mntbuf) < 0) {
 		warning(ENOTDIR, path);
-#ifndef	_NODOSDRIVE
+#ifdef	DEP_DOSDRIVE
 		needbavail--;
 #endif
 		return(0);
@@ -1164,9 +1232,12 @@ CONST char *path;
 	y = info1line(yy, y, FSNAM_K, (off_t)0, mntbuf.mnt_fsname, NULL);
 	y = info1line(yy, y, FSMNT_K, (off_t)0, mntbuf.mnt_dir, NULL);
 	y = info1line(yy, y, FSTYP_K, (off_t)0, mntbuf.mnt_type, NULL);
+#ifndef	NOFBLOCKS
 	y = info1line(yy, y, FSTTL_K,
 		calcKB((off_t)(fsbuf.f_blocks), (off_t)blocksize(fsbuf)),
 		NULL, "Kbytes");
+#endif
+#ifndef	NOFBFREE
 	y = info1line(yy, y, FSUSE_K,
 		calcKB((off_t)(fsbuf.f_blocks - fsbuf.f_bfree),
 			(off_t)blocksize(fsbuf)),
@@ -1174,8 +1245,11 @@ CONST char *path;
 	y = info1line(yy, y, FSAVL_K,
 		calcKB((off_t)(fsbuf.f_bavail), (off_t)blocksize(fsbuf)),
 		NULL, "Kbytes");
+#endif
 	y = info1line(yy, y, FSBSZ_K, (off_t)(fsbuf.f_bsize), NULL, "bytes");
+#ifndef	NOFFILES
 	y = info1line(yy, y, FSINO_K, (off_t)(fsbuf.f_files), NULL, UNIT_K);
+#endif
 	if (y > 1) {
 		for (; y < FILEPERROW; y++) {
 			Xlocate(0, yy + y);
@@ -1183,7 +1257,7 @@ CONST char *path;
 		}
 		warning(0, HITKY_K);
 	}
-#ifndef	_NODOSDRIVE
+#ifdef	DEP_DOSDRIVE
 	needbavail--;
 #endif
 
